﻿-- Enviorment checkup
local LBFIsOK, LBF = pcall(LibStub, "LibButtonFacade",false)

-- Saved Variables
SKINID = nil
GLOSS = nil
BACKDROP = nil
FLASH_KICKIN = 10
FLASH_LOWERVALUE = 0.4
FLASH_UPPERVALUE = 1
FLASH_DOWNTIME = 0.6
FLASH_UPTIME = 0.2
NOTICE = 1
NOTICE2 = 1
ANNOUNCE = 1
ANNOUNCE_TO = "FCT"
TEXT_FONT = "Interface\\AddOns\\MacTotemTimer\\font.ttf"
TEXT_SIZE = 9
TEXT_OUTLINE = 1
TEXT_SHADOW = 1
TEXT_OFFSETY = 8
TEXT_COLOR = {1.0,0.82,0}
UNLOCK_MODE = nil
OFFSET_X = 0
OFFSET_Y = 0
TOTEM1 = 1
TOTEM2 = 2
TOTEM3 = 3
TOTEM4 = 4
BUTTON_SCALE = 1.0
ANNOUNCE_COLOR = {r = 0, g = 0.8*255, b = 1*255}

--[[ exception
-- @autor: Christoph Bähr
-- @description: exception handler
-- @args: message: exception message
--]]
function exception(message)
 _ERRORMESSAGE(message)
 return nil
end 

--[[ callback
-- @autor: Christoph Bähr
-- @args:  arg,SkinID,Gloss,Backdrop,Group,Button,Colors
--]]
function callback(arg,skinID,gloss,backdrop,Group,Button,Colors)
 SKINID = skinID
 GLOSS = gloss
 BACKDROP = backdrop
end

--[[ flash
--@autor: Christoph Bähr
--@description: flash effect for given Button
--@args: tempObject Button
--]]
function flash(tempObject)
 if tempObject:GetAlpha() >= FLASH_LOWERVALUE and downFlash == true then 
  tempObject:SetAlpha(tempObject:GetAlpha() - FLASH_DOWNTIME) -- fall
 end
 
 if tempObject:GetAlpha() <= FLASH_LOWERVALUE then -- turn up
  downFlash = false
  upFlash = true
 end
 if tempObject:GetAlpha() <= FLASH_UPPERVALUE and upFlash == true then -- raise
  tempObject:SetAlpha(tempObject:GetAlpha() + FLASH_UPTIME)
 end
 if tempObject:GetAlpha() >= FLASH_UPPERVALUE then -- turn down
  upFlash = false
  downFlash = true
 end
end

--[[ totemFunc
--@autor: Christoph Bähr
--@description: shared function for OnUpdate and OnEvent
--@args: tempObject Button
--]]
function totemFunc(tempObject)
   tempObject.enabled, tempObject.totemName, tempObject.startTime, tempObject.duration, tempObject.filePath = GetTotemInfo(tempObject.id)
    tempObject.elapsedTime = GetTotemTimeLeft(tempObject.id)
    if tempObject.totemName ~= "" then
	 tempObject:Show()
	 if tempObject.elapsedTime > 0 then
	  tempObject.notice2 = false
	 end
     if tempObject.elapsedTime > FLASH_KICKIN then
	  tempObject:SetAlpha(1)
	  tempObject.flash = false
	  tempObject.notice = false
	 end
	 tempObject.icon:SetTexture(tempObject.filePath)
 	 
	  --Time formatting
     if tempObject.elapsedTime / 60 >= 1 then -- there are minutes left
	  tempObject:SetText(nil)
     else -- there are only few seconds
	  tempObject:SetFormattedText("%i",tempObject.elapsedTime)
	  local tempText = tempObject:GetFontString()
	  if tempText then
	   tempText:SetPoint("BOTTOM", tempObject, "BOTTOM", 0, TEXT_OFFSETY)
	  end
     end
   
     if tempObject.flash == true then
	  flash(tempObject)
	 end
	 
    if tempObject.elapsedTime <= 0 then -- Time-based reconfiguration
     if NOTICE2 == 1 and not tempObject.notice2 then
	  tempObject.notice2 = true
	  announce(tempObject.totemName, "destroyed")
	 end
	 tempObject:SetText(nil)
 	 tempObject:SetAlpha(0)
	 tempObject.flash = false
    elseif tempObject.elapsedTime <= FLASH_KICKIN then
     tempObject.flash = true
	 if NOTICE == 1 and not tempObject.notice then
	  tempObject.notice = true
	  announce(tempObject.totemName) -- see announcer.lua
	 end
    else
	 tempObject:SetFont(TEXT_FONT, TEXT_SIZE)
	 if TEXT_OUTLINE == 1 then
	  tempObject:SetFont(TEXT_FONT, TEXT_SIZE, "OUTLINE")
	 end
	 tempObject:SetTextColor(unpack(TEXT_COLOR))
	 if TEXT_SHADOW == 1 then
	  local tempText = tempObject:GetFontString()
	  if tempText then
	   tempText:SetShadowOffset(1, -1)
	   tempText:SetShadowColor(0, 0, 0, 1)
	  end
	 else
	  local tempText = tempObject:GetFontString()
	  if tempText then
	   tempText:SetShadowOffset(0, 0)
	   tempText:SetShadowColor(0, 0, 0, 0)
	  end
	 end
    end
  else
   tempObject:Hide()
  end
end

--[[ totemFactory
-- @autor: Christoph Bähr
-- @description: factory for totembuttons
-- @args: totemID describe the current identification to represent a totem; Bounds(X, Y, width, height)
--]]
function totemFactory(totemID, X, Y, width, height)
 if macTotemTimerFrame ~= nil then
  local tempObject = CreateFrame("Button", "Totem"..totemID , macTotemTimerFrame)
  tempObject.gloss = tempObject:CreateTexture("Totem"..totemID.."Gloss", "OVERLAY")
  tempObject.backdrop = tempObject:CreateTexture("Totem"..totemID.."Backdrop", "BACKGROUND")
  tempObject.icon = tempObject:CreateTexture("Totem"..totemID.."Icon", "ARTWORK")
  
  tempObject.icon:SetAllPoints(tempObject)
  tempObject.gloss:SetAllPoints(tempObject)
  tempObject.backdrop:SetAllPoints(tempObject)
    
  tempObject:SetPoint("TOPLEFT", macTotemTimerFrame, "TOPLEFT", X, Y)
  tempObject:SetWidth(width)
  tempObject:SetHeight(height)
  tempObject.startTime = 0
  tempObject.duration = 0
  tempObject.flash = false
  tempObject.id = totemID
  tempObject.downFlash = true
  tempObject.upFlash = false
  tempObject:SetPushedTextOffset(0,0)
  tempObject.notice = false 
  tempObject.notice2 = false
  
  tempObject:SetScript("OnMouseDown", function(frame, mouseButton)
   if mouseButton == "RightButton" then -- destroy totem
    DestroyTotem(totemID)
   end
  end)
  
  tempObject:SetScript("OnEvent", function(frame, event, slot)
   totemFunc(tempObject)
  end)
  
  tempObject:RegisterEvent("PLAYER_TOTEM_UPDATE")
  
  tempObject:SetScript("OnUpdate", function(frame)
    totemFunc(tempObject)
   end)
  
  return tempObject
 else
  return exception("Parent for this Totem was not set.")
 end
end

--[[ prepareWindow
-- @autor: Christoph Bähr
-- @description: decorate the macTotemTimerFrame
-- @args: none
--]]
function prepareWindow()
  macTotemTimerFrame:SetWidth(156)
  macTotemTimerFrame:SetHeight(32)
  macTotemTimerFrame:EnableMouse(true)
  macTotemTimerFrame:SetMovable(true)
  macTotemTimerFrame:SetUserPlaced(true)
  macTotemTimerFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = nil, 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            insets = { left = 0, right = 0, top = 0, bottom = 0 }});
 
  macTotemTimerFrame:SetScript("OnEnter", function()
   if UNLOCK_MODE then
   macTotemTimerFrame:SetBackdropColor(0,0,0,0.5);
   end
  end)
  macTotemTimerFrame:SetScript("OnLeave", function()
   if UNLOCK_MODE then
    macTotemTimerFrame:SetBackdropColor(0,0,0,0.3);
   else
    macTotemTimerFrame:SetBackdropColor(0,0,0,0);
   end
  end)
  macTotemTimerFrame:SetScript("OnMouseDown", function(frame, mouseButton)
   if mouseButton == "RightButton" and UNLOCK_MODE then
    macTotemTimerFrame:StartMoving()
   end
  end)
  macTotemTimerFrame:SetScript("OnMouseUp", function(frame, mouseButton)
   if mouseButton == "RightButton" then
    macTotemTimerFrame:StopMovingOrSizing()
   end
  end)
  
  macTotemTimerFrame:SetPoint("CENTER", UIParent)
end

--[[ main
-- @autor: Christoph Bähr
-- @description: main function for totem timer (World of Warcraft)
-- @args: none
--]]
function main()
  macTotemTimerFrame = CreateFrame("Frame", "macTotemTimerFrame", UIParent)
  prepareWindow()

  Totem1 = totemFactory(TOTEM1, 0, 0, 32, 32)
  Totem2 = totemFactory(TOTEM2, 36, 0, 32, 32)
  Totem3 = totemFactory(TOTEM3, 72, 0, 32, 32)
  Totem4 = totemFactory(TOTEM4, 108, 0, 32, 32)
  
  if LBFIsOK then
   local group = LBF:Group("MacTotemTimer")
   group:AddButton(Totem1)
   group:AddButton(Totem2)
   group:AddButton(Totem3)
   group:AddButton(Totem4)
   LBF:RegisterSkinCallback("MacTotemTimer", callback, self) 
  end
  
  macTotemTimerFrame:SetScript("OnEvent", function(frame, event, ...)
   if event == "VARIABLES_LOADED" then
    Totem1.id = TOTEM1
	Totem2.id = TOTEM2
	Totem3.id = TOTEM3
	Totem4.id = TOTEM4
	macTotemTimerFrame:SetScale(BUTTON_SCALE)
	if UNLOCK_MODE == nil then
	 macTotemTimerFrame:SetBackdropColor(0,0,0,0);
	else
	 macTotemTimerFrame:SetBackdropColor(0,0,0,0.3);
	end
    if SKINID ~= nil and LBFIsOK then -- previous settings do exists
	 LBF:Group("MacTotemTimer"):Skin(SKINID, GLOSS, BACKDROP)
	end
	if TEXT_FONT == nil then
	 TEXT_FONT = "Interface\\AddOns\\MacTotemTimer\\font.ttf"
	 FLASH_LOWERVALUE = 0.4
	 FLASH_UPPERVALUE = 1
	 FLASH_DOWNTIME = 0.02
	 FLASH_UPTIME = 0.06

	 TEXT_SIZE = 9
	 TEXT_OUTLINE = true
	 TEXT_SHADOW = true
     UNLOCK_MODE = false
	 HIDE_BLIZZARD = false
	end
	setupUIPanel() -- @ see also : Settings.lua
    if HIDE_BLIZZARD == 1 then
     TotemFrame:UnregisterAllEvents()
	 TotemFrame:Hide()
    end
   end
  end)  

  macTotemTimerFrame:RegisterEvent("VARIABLES_LOADED")
end 

main() 